﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class BuildEngineer : EngineerModule
    {
        ThrustToWeight ttw;

        Vector3 mousePos;
        bool drawStandard = true;
        bool drawThrust = false;
        bool drawAllStages = false;

        public override void OnStart(StartState state)
        {
            base.OnStart(state);

            if (state == StartState.Editor)
            {
                this.part.OnEditorAttach = OnEditorAttach;
                this.part.OnEditorDetach = OnEditorDetach;
                this.part.OnEditorDestroy = OnEditorDestroy;

                OnEditorAttach();

                Debug("BuildEngineer: OnStart");
            }
        }

        public void OnEditorAttach()
        {
            this.IsPrimary = CheckIsPrimary("BuildEngineer", EditorLogic.SortedShipList);

            if (IsPrimary)
            {
                LoadSettings();

                if (winPos.x == 0 && winPos.y == 0)
                    winPos = new Rect(300, 70, 0, 0);

                ttw = new ThrustToWeight();
                ship.Update(EditorLogic.SortedShipList);
                RenderingManager.AddToPostDrawQueue(0, DrawGUI);

                Debug("BuildEngineer: OnEditorAttach");
            }
        }

        public void OnEditorDetach()
        {
            if (IsPrimary)
            {
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);

                Debug("BuildEngineer: OnEditorDetach");
            }

        }

        public void OnEditorDestroy()
        {
            if (IsPrimary)
            {
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);

                Debug("BuildEngineer: OnEditorDestroy");
            }
        }

        public override void DrawGUI()
        {
            base.DrawGUI();

            if (this.part == null)
            {
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
                return;
            }

            Rect winPosBefore = winPos;
            winPos = GUILayout.Window(3, winPos, MainWindow, "Kerbal Engineer Redux - Version " + Version, winStyle);
            if (winPos != winPosBefore) SaveSettings();

            mousePos = Input.mousePosition;
            mousePos.y = Screen.height - mousePos.y;
            if (winPos.Contains(mousePos))
            {
                EditorLogic.softLock = true;
            }
            else
            {
                EditorLogic.softLock = false;
            }

            //Debug("BuildEngineer: DrawGUI");
        }

        protected override void InitGUI()
        {
            base.InitGUI();
            winStyle.fixedWidth = 700;
        }

        public void MainWindow(int windowId)
        {
            GUILayout.BeginHorizontal();
            string stdButtonText = "Hide Standard Specification";
            if (!drawStandard)
                stdButtonText = "Show Standard Specification";

            string ttwButtonText = "Hide Thrust to Weight Ratios";
            if (!drawThrust)
                ttwButtonText = "Show Thrust to Weight Ratios";

            string stagesButtonText = "All Stages";
            if (drawAllStages)
                stagesButtonText = "Important Stages";

            buttonStyle.fixedWidth = 250;
            if (GUILayout.Button(stdButtonText, buttonStyle))
            {
                drawStandard = !drawStandard;
                winPos.height = 0;
            }

            if (GUILayout.Button(ttwButtonText, buttonStyle))
            {
                drawThrust = !drawThrust;
                winPos.height = 0;
            }

            buttonStyle.fixedWidth = 0;
            if (GUILayout.Button(stagesButtonText, buttonStyle))
            {
                drawAllStages = !drawAllStages;
                winPos.height = 0;
            }
            GUILayout.EndHorizontal();

            if (drawStandard)
                DrawStandard();

            if (drawThrust)
                DrawThrust();

            //GUILayout.Label(EditorLogic.softLock.ToString(), GUIStyle.none);

            GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        private void DrawStandard()
        {
            GUILayout.BeginHorizontal();
            GUILayout.Label("Standard Specifications", titleStyle);
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);

            GUILayout.BeginVertical();
            GUILayout.Label("", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label("S " + Convert.ToString(i), headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Cost", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].Cost.ToString("0") + " / " + ship.Stages[i].TotalCost.ToString("0"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Mass", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].Mass.ToString("0.00") + " / " + ship.Stages[i].TotalMass.ToString("0.00"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Fuel", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].Fuel.ToString("0"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Isp(A/V)", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].AtmIsp.ToString("0") + " / " + ship.Stages[i].VacIsp.ToString("0"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Thrust", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].Thrust.ToString("0"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].DeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.EndVertical();

            GUILayout.EndHorizontal();
        }

        private void DrawThrust()
        {
            GUILayout.BeginHorizontal();
            GUILayout.Label("Thrust to Weight Ratios", titleStyle);
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);

            GUILayout.BeginVertical();
            GUILayout.Label("", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label("S " + Convert.ToString(i), headStyle);
            GUILayout.EndVertical();

            foreach (string influenceName in ttw.GravityList.Keys.ToArray<string>())
            {
                GUILayout.BeginVertical();
                GUILayout.Label(influenceName, headStyle);
                for (int i = 0; i < ship.Stages.Count; i++)
                    if (ship.Stages[i].Useful || drawAllStages)
                        GUILayout.Label((ship.Stages[i].Thrust / (ship.Stages[i].TotalMass * ttw.GravityList[influenceName])).ToString("0.00"), dataStyle);
                GUILayout.EndVertical();
            }

            GUILayout.EndHorizontal();
        }

        public override void OnSave(ConfigNode node)
        {
            base.OnSave(node);

            if (state == StartState.Editor && IsPrimary)
            {
                ship.Update(EditorLogic.SortedShipList);
                winPos.height = 0;

                Debug("BuildEngineer: OnSave");
            }
        }

        public override void OnLoad(ConfigNode node)
        {
            base.OnLoad(node);

            LoadSettings();
        }

        private void SaveSettings()
        {
            settings.SetValue("BuildEngineer::winPos.x", winPos.x.ToString());
            settings.SetValue("BuildEngineer::winPos.y", winPos.y.ToString());
            settings.Save();
        }

        private void LoadSettings()
        {
            settings.Load();
            winPos.x = Convert.ToSingle(settings.GetValue("BuildEngineer::winPos.x"));
            winPos.y = Convert.ToSingle(settings.GetValue("BuildEngineer::winPos.y"));
        }
    }
}
