﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class EngineerModule : PartModule
    {
        public Settings settings = new Settings();
        public StartState state = StartState.None;
        public Ship ship;
        public Rect winPos;
        public GUIStyle winStyle, areaStyle, titleStyle, headStyle, dataStyle, buttonStyle, scrollStyle;
        public bool hasInitGUI = false;

        public string Version
        {
            get { return "0.4.3"; }
        }

        public bool IsPrimary
        {
            get;
            protected set;
        }

        public Stage GetStage(int number)
        {
            if (number < ship.Stages.Count)
            {
                return ship.Stages[number];
            }
            else
            {
                return ship.Stages[ship.Stages.Count - 1];
            }
        }

        public override void OnStart(StartState state)
        {
            base.OnStart(state);
            this.state = state;

            ship = new Ship(this);

            Debug("EngineerModule: OnStart(" + state + ")");
        }

        public override void OnSave(ConfigNode node)
        {
            foreach (ConfigNode.Value value in node.values)
            {
                Debug("EngineerModule (Node): " + value.name + " = " + value.value);
            }
            Debug("EngineerModule: OnSave");
        }

        public override void OnLoad(ConfigNode node)
        {
            foreach (ConfigNode.Value value in node.values)
            {
                Debug("EngineerModule (Node): " + value.name + " = " + value.value);
            }
            Debug("EngineerModule: OnLoad");
        }

        protected bool CheckIsPrimary(string className, List<Part> partList)
        {
            if (this.part.parent == null)
                return false;

            foreach (Part part in partList)
            {
                if (part != this.part && part.Modules.Contains(className))
                {
                    EngineerModule partMod = (part.Modules[className] as EngineerModule);
                    if (partMod.IsPrimary)
                    {
                        Debug("EngineerModule: CheckIsPrimary(" + className + ") = false");
                        return false;
                    }
                }
            }

            Debug("EngineerModule: CheckIsPrimary(" + className + ") = true");
            return true;
        }

        virtual public void DrawGUI()
        {
            if (!hasInitGUI)
                InitGUI();
        }

        virtual protected void InitGUI()
        {
            GUI.skin = HighLogic.Skin;

            winStyle = new GUIStyle(GUI.skin.window);
            winStyle.stretchHeight = true;

            areaStyle = new GUIStyle(GUI.skin.box);
            areaStyle.hover = areaStyle.active = areaStyle.normal;

            titleStyle = new GUIStyle(GUI.skin.label);
            titleStyle.normal.textColor = Color.yellow;
            titleStyle.fontStyle = FontStyle.Bold;

            headStyle = new GUIStyle(GUI.skin.label);
            headStyle.stretchWidth = true;
            headStyle.normal.textColor = Color.white;
            headStyle.fontStyle = FontStyle.Bold;
            headStyle.wordWrap = false;

            dataStyle = new GUIStyle(GUI.skin.label);
            dataStyle.stretchWidth = true;
            dataStyle.wordWrap = false;

            buttonStyle = new GUIStyle(GUI.skin.button);
            buttonStyle.stretchWidth = true;

            scrollStyle = new GUIStyle(GUI.skin.scrollView);

            hasInitGUI = true;

            Debug("EngineerModule: InitGUI");
        }

        [Conditional("DEBUG")]
        public void Debug(string message)
        {
            print(message);
        }
    }
}
